@@ -1,22 +1,16 @@
 // SPDX-FileCopyrightText: 2021 Claudio Cambra <claudio.cambra@gmail.com>
 // SPDX-License-Identifier: LGPL-2.1-or-later
 
+#include "kalendar_debug.h"
 #include <KLocalizedString>
 #include <QBitArray>
 #include <QJSValue>
-#include <QMetaEnum>
 #include <incidencewrapper.h>
 
 IncidenceWrapper::IncidenceWrapper(QObject *parent)
     : QObject(parent)
-    , KCalendarCore::IncidenceBase::IncidenceObserver()
-    , m_incidence(new KCalendarCore::Event)
-    , m_remindersModel(parent, m_incidence)
-    , m_attendeesModel(parent, m_incidence)
-    , m_recurrenceExceptionsModel(parent, m_incidence)
-    , m_attachmentsModel(parent, m_incidence)
+    , Akonadi::ItemMonitor()
 {
-    m_incidence->registerObserver(this);
     // Change incidence pointer in remindersmodel if changed here
     connect(this, &IncidenceWrapper::incidencePtrChanged, &m_remindersModel, [=](KCalendarCore::Incidence::Ptr incidencePtr) {
         m_remindersModel.setIncidencePtr(incidencePtr);
@@ -30,26 +24,18 @@ IncidenceWrapper::IncidenceWrapper(QObject *parent)
     connect(this, &IncidenceWrapper::incidencePtrChanged, &m_attachmentsModel, [=](KCalendarCore::Incidence::Ptr incidencePtr) {
         m_attachmentsModel.setIncidencePtr(incidencePtr);
     });
+
+    Akonadi::ItemFetchScope scope;
+    scope.fetchFullPayload();
+    scope.fetchAllAttributes();
+    scope.setFetchRelations(true);
+    scope.setAncestorRetrieval(Akonadi::ItemFetchScope::Parent);
+    setFetchScope(scope);
+
+    setNewEvent();
 }
 
-IncidenceWrapper::~IncidenceWrapper()
-{
-    m_incidence->unRegisterObserver(this);
-}
-
-void IncidenceWrapper::incidenceUpdate(const QString &uid, const QDateTime &)
-{
-    if (uid == m_incidence->uid()) {
-        Q_EMIT incidenceAboutToChange();
-    }
-}
-
-void IncidenceWrapper::incidenceUpdated(const QString &uid, const QDateTime &)
-{
-    if (uid == m_incidence->uid()) {
-        notifyDataChanged();
-    }
-}
+IncidenceWrapper::~IncidenceWrapper() = default;
 
 void IncidenceWrapper::notifyDataChanged()
 {
@@ -83,6 +69,23 @@ void IncidenceWrapper::notifyDataChanged()
     Q_EMIT todoPercentCompleteChanged();
 }
 
+Akonadi::Item IncidenceWrapper::incidenceItem() const
+{
+    return item();
+}
+
+void IncidenceWrapper::setIncidenceItem(const Akonadi::Item &incidenceItem)
+{
+    if (incidenceItem.hasPayload<KCalendarCore::Incidence::Ptr>()) {
+        setItem(incidenceItem);
+        setIncidencePtr(incidenceItem.payload<KCalendarCore::Incidence::Ptr>());
+        Q_EMIT incidenceItemChanged();
+        Q_EMIT collectionIdChanged();
+    } else {
+        qCWarning(KALENDAR_LOG) << "This is not an incidence item.";
+    }
+}
+
 KCalendarCore::Incidence::Ptr IncidenceWrapper::incidencePtr() const
 {
     return m_incidence;
@@ -90,9 +93,7 @@ KCalendarCore::Incidence::Ptr IncidenceWrapper::incidencePtr() const
 
 void IncidenceWrapper::setIncidencePtr(const KCalendarCore::Incidence::Ptr incidencePtr)
 {
-    m_incidence->unRegisterObserver(this);
     m_incidence = incidencePtr;
-    m_incidence->registerObserver(this);
 
     KCalendarCore::Incidence::Ptr originalIncidence(incidencePtr->clone());
     m_originalIncidence = originalIncidence;
@@ -129,7 +130,7 @@ QString IncidenceWrapper::uid() const
 
 qint64 IncidenceWrapper::collectionId() const
 {
-    return m_collectionId;
+    return m_collectionId < 0 ? item().parentCollection().id() : m_collectionId;
 }
 
 void IncidenceWrapper::setCollectionId(qint64 collectionId)
@@ -228,6 +229,9 @@ void IncidenceWrapper::setIncidenceStart(const QDateTime &incidenceStart, bool r
 
     if (respectTimeZone) {
         m_incidence->setDtStart(incidenceStart);
+        auto newTzEnd = incidenceEnd();
+        newTzEnd.setTimeZone(incidenceStart.timeZone());
+        setIncidenceEnd(newTzEnd, true);
     } else {
         const auto date = incidenceStart.date();
         const auto time = incidenceStart.time();
@@ -311,7 +315,7 @@ void IncidenceWrapper::setIncidenceEnd(const QDateTime &incidenceEnd, bool respe
         KCalendarCore::Todo::Ptr todo = m_incidence.staticCast<KCalendarCore::Todo>();
         todo->setDtDue(end);
     } else {
-        qWarning() << "Unknown incidence type";
+        qCWarning(KALENDAR_LOG) << "Unknown incidence type";
     }
     Q_EMIT incidenceEndChanged();
     Q_EMIT incidenceEndDateDisplayChanged();
@@ -460,7 +464,7 @@ void IncidenceWrapper::setRecurrenceDataItem(const QString &key, const QVariant
                 return;
             }
 
-            QVariantList vlist = jsval.toVariant().value<QVariantList>();
+            auto vlist = jsval.toVariant().value<QVariantList>();
             QBitArray days(7);
 
             for (int i = 0; i < vlist.size(); i++) {
@@ -592,7 +596,7 @@ void IncidenceWrapper::setTodoCompleted(bool completed)
 QDateTime IncidenceWrapper::todoCompletionDt()
 {
     if (m_incidence->type() != KCalendarCore::IncidenceBase::TypeTodo) {
-        return QDateTime();
+        return {};
     }
 
     auto todo = m_incidence.staticCast<KCalendarCore::Todo>();
@@ -633,18 +637,31 @@ void IncidenceWrapper::triggerEditMode() // You edit a clone so that the origina
     setIncidencePtr(clonedPtr);
 }
 
+static int nearestQuarterHour(int secsSinceEpoch)
+{
+    const int quarterHourInSecs = 60 * 15;
+    return secsSinceEpoch + (quarterHourInSecs - secsSinceEpoch % quarterHourInSecs);
+}
+
 void IncidenceWrapper::setNewEvent()
 {
     auto event = KCalendarCore::Event::Ptr(new KCalendarCore::Event);
-    event->setDtStart(QDateTime::currentDateTime());
-    event->setDtEnd(QDateTime::currentDateTime().addSecs(60 * 60));
-    setIncidencePtr(event);
+    QDateTime start;
+    start.setSecsSinceEpoch(nearestQuarterHour(QDateTime::currentSecsSinceEpoch()));
+    event->setDtStart(start);
+    event->setDtEnd(start.addSecs(60 * 60));
+
+    Akonadi::Item incidenceItem;
+    incidenceItem.setPayload<KCalendarCore::Event::Ptr>(event);
+    setIncidenceItem(incidenceItem);
 }
 
 void IncidenceWrapper::setNewTodo()
 {
     auto todo = KCalendarCore::Todo::Ptr(new KCalendarCore::Todo);
-    setIncidencePtr(todo);
+    Akonadi::Item incidenceItem;
+    incidenceItem.setPayload<KCalendarCore::Todo::Ptr>(todo);
+    setIncidenceItem(incidenceItem);
 }
 
 void IncidenceWrapper::addAlarms(KCalendarCore::Alarm::List alarms)
@@ -674,7 +691,7 @@ void IncidenceWrapper::setRegularRecurrence(IncidenceWrapper::RecurrenceInterval
         Q_EMIT recurrenceDataChanged();
         return;
     default:
-        qWarning() << "Unknown interval for recurrence" << interval;
+        qCWarning(KALENDAR_LOG) << "Unknown interval for recurrence" << interval;
         return;
     }
 }
@@ -698,4 +715,12 @@ void IncidenceWrapper::clearRecurrences()
     Q_EMIT recurrenceDataChanged();
 }
 
+void IncidenceWrapper::itemChanged(const Akonadi::Item &item)
+{
+    if (item.hasPayload<KCalendarCore::Incidence::Ptr>()) {
+        qCDebug(KALENDAR_LOG) << item.payload<KCalendarCore::Incidence::Ptr>()->summary() << item.parentCollection().id();
+        setIncidencePtr(item.payload<KCalendarCore::Incidence::Ptr>());
+    }
+}
+
 Q_DECLARE_METATYPE(KCalendarCore::Incidence::Ptr);
