@@ -23,8 +23,13 @@ InfiniteCalendarViewModel::InfiniteCalendarViewModel(QObject *parent)
     ModelMetaData scheduleModel = {QVector<QDate>(), 0, TypeSchedule, &m_scheduleViewModels, {}, &m_liveScheduleViewModelKeys};
     ModelMetaData weekModel = {QVector<QDate>(), 7, TypeWeek, {}, &m_weekViewModels, &m_liveWeekViewModelKeys};
     ModelMetaData weekMultiDayModel = {QVector<QDate>(), 7, TypeWeekMultiDay, &m_weekViewMultiDayModels, {}, &m_liveWeekViewMultiDayModelKeys};
+    ModelMetaData threeDayModel = {QVector<QDate>(), 7, TypeThreeDay, {}, &m_threeDayViewModels, &m_liveThreeDayViewModelKeys};
+    ModelMetaData threeDayMultiDayModel = {QVector<QDate>(), 7, TypeThreeDayMultiDay, &m_threeDayViewMultiDayModels, {}, &m_liveThreeDayViewMultiDayModelKeys};
+    ModelMetaData dayModel = {QVector<QDate>(), 7, TypeDay, {}, &m_dayViewModels, &m_liveDayViewModelKeys};
+    ModelMetaData dayMultiDayModel = {QVector<QDate>(), 7, TypeDayMultiDay, &m_dayViewMultiDayModels, {}, &m_liveDayViewMultiDayModelKeys};
 
-    m_models = QVector<ModelMetaData>{monthModel, scheduleModel, weekModel, weekMultiDayModel};
+    m_models =
+        QVector<ModelMetaData>{monthModel, scheduleModel, weekModel, weekMultiDayModel, threeDayModel, threeDayMultiDayModel, dayModel, dayMultiDayModel};
 }
 
 void InfiniteCalendarViewModel::setup()
@@ -32,15 +37,29 @@ void InfiniteCalendarViewModel::setup()
     const QDate today = QDate::currentDate();
     QTime time;
 
-    if (!m_weekViewLocalisedHourLabels.length()) {
-        m_weekViewLocalisedHourLabels.clear();
+    if (!m_hourlyViewLocalisedHourLabels.length()) {
+        m_hourlyViewLocalisedHourLabels.clear();
         for (int i = 1; i < 24; i++) {
             time.setHMS(i, 0, 0);
-            m_weekViewLocalisedHourLabels.append(QLocale::system().toString(time, QLocale::NarrowFormat));
+            m_hourlyViewLocalisedHourLabels.append(QLocale::system().toString(time, QLocale::NarrowFormat));
         }
     }
 
     switch (m_scale) {
+    case DayScale: {
+        QDate firstDay = today;
+        firstDay = firstDay.addDays(m_datesToAdd / 2);
+
+        addDayDates(true, firstDay);
+        break;
+    }
+    case ThreeDayScale: {
+        QDate firstDay = today;
+        firstDay = firstDay.addDays((-m_datesToAdd * 3) / 2);
+
+        addDayDates(true, firstDay, 3);
+        break;
+    }
     case WeekScale: {
         QDate firstDay = today.addDays(-today.dayOfWeek() + m_locale.firstDayOfWeek());
         // We create dates before and after where our view will start from (which is today)
@@ -94,9 +113,23 @@ QVariant InfiniteCalendarViewModel::data(const QModelIndex &idx, int role) const
         return model;
     };
 
+    auto generateHourlyIncidenceModel = [&](QDate start, int length, int periodLength) {
+        auto model = new HourlyIncidenceModel;
+        model->setPeriodLength(periodLength);
+        model->setFilters(HourlyIncidenceModel::NoAllDay | HourlyIncidenceModel::NoMultiDay);
+        model->setModel(new IncidenceOccurrenceModel);
+        model->model()->setStart(start);
+        model->model()->setLength(length);
+        model->model()->setFilter(mFilter);
+        model->model()->setCalendar(m_calendar);
+
+        return model;
+    };
+
     auto cleanUpModels = [&, this]() {
         int numLiveModels = m_liveMonthViewModelKeys.length() + m_liveScheduleViewModelKeys.length() + m_liveWeekViewModelKeys.length()
-            + m_liveWeekViewMultiDayModelKeys.length();
+            + m_liveWeekViewMultiDayModelKeys.length() + m_liveThreeDayViewModelKeys.length() + m_liveThreeDayViewMultiDayModelKeys.length()
+            + m_liveDayViewModelKeys.length() + m_liveDayViewMultiDayModelKeys.length(); // Find a more elegant way to do this
 
         while (numLiveModels > m_maxLiveModels) {
             for (int i = 0; i < m_models.length(); i++) {
@@ -104,34 +137,42 @@ QVariant InfiniteCalendarViewModel::data(const QModelIndex &idx, int role) const
                     while (m_models[i].liveKeysQueue->length() > m_maxLiveModels) {
                         auto firstKey = m_models[i].liveKeysQueue->dequeue();
 
-                        if (m_models[i].modelType == TypeWeek && m_models[i].weekModels->contains(firstKey)) {
-                            delete m_models[i].weekModels->value(firstKey);
-                            m_models[i].weekModels->remove(firstKey);
+                        if ((m_models[i].modelType == TypeWeek || m_models[i].modelType == TypeThreeDay || m_models[i].modelType == TypeDay)
+                            && m_models[i].hourlyModels->contains(firstKey)) {
+                            delete m_models[i].hourlyModels->value(firstKey);
+                            m_models[i].hourlyModels->remove(firstKey);
                         } else if (m_models[i].multiDayModels->contains(firstKey)) {
                             delete m_models[i].multiDayModels->value(firstKey);
                             m_models[i].multiDayModels->remove(firstKey);
                         }
                     }
 
-                } else if(m_models[i].modelType == m_lastAccessedModelType/* ||
-                   ((m_lastAccessedModelType == TypeWeek && m_models[i].modelType == TypeWeekMultiDay) ||
-                   (m_lastAccessedModelType == TypeWeekMultiDay && m_models[i].modelType == TypeWeek))*/) {
-                    continue;
-
                 } else if (m_models[i].liveKeysQueue->length() > 0) {
                     auto firstKey = m_models[i].liveKeysQueue->dequeue();
 
-                    if (m_models[i].modelType == TypeWeek && m_models[i].weekModels->contains(firstKey)) {
-                        delete m_models[i].weekModels->value(firstKey);
-                        m_models[i].weekModels->remove(firstKey);
+                    if ((m_models[i].modelType == TypeWeek || m_models[i].modelType == TypeThreeDay || m_models[i].modelType == TypeDay)
+                        && m_models[i].hourlyModels->contains(firstKey)) {
+                        delete m_models[i].hourlyModels->value(firstKey);
+                        m_models[i].hourlyModels->remove(firstKey);
                     } else if (m_models[i].multiDayModels->contains(firstKey)) {
                         delete m_models[i].multiDayModels->value(firstKey);
                         m_models[i].multiDayModels->remove(firstKey);
                     }
                 }
 
+
                 numLiveModels = m_liveMonthViewModelKeys.length() + m_liveScheduleViewModelKeys.length() + m_liveWeekViewModelKeys.length()
-                    + m_liveWeekViewMultiDayModelKeys.length();
+                    + m_liveWeekViewMultiDayModelKeys.length() + m_liveThreeDayViewModelKeys.length() + m_liveThreeDayViewMultiDayModelKeys.length()
+                    + m_liveDayViewModelKeys.length() + m_liveDayViewMultiDayModelKeys.length();
+            }
+        }
+    };
+
+    auto requeue = [&, this](QQueue<QDate> &liveKeysQueue, const QDate &key) {
+        for (int i = 0; i < liveKeysQueue.length(); i++) {
+            if (liveKeysQueue[i] == key) {
+                liveKeysQueue.move(i, liveKeysQueue.length() - 1);
+                break;
             }
         }
     };
@@ -154,11 +195,13 @@ QVariant InfiniteCalendarViewModel::data(const QModelIndex &idx, int role) const
                 // Unfortunately this gets called by the pathviews no matter what the currentIndex
                 // value is set to.
             }
-            if (!m_monthViewModels.contains(startDate)) {
+            if (!m_liveMonthViewModelKeys.contains(startDate)) {
                 m_monthViewModels[startDate] = generateMultiDayIncidenceModel(startDate, 42, 7);
 
                 m_liveMonthViewModelKeys.enqueue(startDate);
                 cleanUpModels();
+            } else {
+                requeue(m_liveMonthViewModelKeys, startDate);
             }
 
             return QVariant::fromValue(m_monthViewModels[startDate]);
@@ -175,6 +218,8 @@ QVariant InfiniteCalendarViewModel::data(const QModelIndex &idx, int role) const
 
                 m_liveScheduleViewModelKeys.enqueue(startDate);
                 cleanUpModels();
+            } else {
+                requeue(m_liveScheduleViewModelKeys, startDate);
             }
 
             return QVariant::fromValue(m_scheduleViewModels[firstDay]);
@@ -200,17 +245,12 @@ QVariant InfiniteCalendarViewModel::data(const QModelIndex &idx, int role) const
         }
 
         if (!m_weekViewModels.contains(startDate)) {
-            m_weekViewModels[startDate] = new HourlyIncidenceModel;
-            m_weekViewModels[startDate]->setPeriodLength(7);
-            m_weekViewModels[startDate]->setFilters(HourlyIncidenceModel::NoAllDay | HourlyIncidenceModel::NoMultiDay);
-            m_weekViewModels[startDate]->setModel(new IncidenceOccurrenceModel);
-            m_weekViewModels[startDate]->model()->setStart(startDate);
-            m_weekViewModels[startDate]->model()->setLength(7);
-            m_weekViewModels[startDate]->model()->setFilter(mFilter);
-            m_weekViewModels[startDate]->model()->setCalendar(m_calendar);
+            m_weekViewModels[startDate] = generateHourlyIncidenceModel(startDate, 7, 15);
 
             m_liveWeekViewModelKeys.enqueue(startDate);
             cleanUpModels();
+        } else {
+            requeue(m_liveWeekViewModelKeys, startDate);
         }
 
         return QVariant::fromValue(m_weekViewModels[startDate]);
@@ -228,10 +268,86 @@ QVariant InfiniteCalendarViewModel::data(const QModelIndex &idx, int role) const
 
             m_liveWeekViewMultiDayModelKeys.enqueue(startDate);
             cleanUpModels();
+        } else {
+            requeue(m_liveWeekViewMultiDayModelKeys, startDate);
         }
 
         return QVariant::fromValue(m_weekViewMultiDayModels[startDate]);
     }
+    case ThreeDayViewModelRole: {
+        m_lastAccessedModelType = TypeThreeDay;
+
+        if (m_datesToAdd > 5 && idx.row() < 2 && m_threeDayViewModels.count() < 3) {
+            return {};
+        }
+
+        if (!m_threeDayViewModels.contains(startDate)) {
+            m_threeDayViewModels[startDate] = generateHourlyIncidenceModel(startDate, 3, 15);
+
+            m_liveThreeDayViewModelKeys.enqueue(startDate);
+            cleanUpModels();
+        } else {
+            requeue(m_liveThreeDayViewModelKeys, startDate);
+        }
+
+        return QVariant::fromValue(m_threeDayViewModels[startDate]);
+    }
+    case ThreeDayViewMultiDayModelRole: {
+        m_lastAccessedModelType = TypeThreeDayMultiDay;
+
+        if (m_datesToAdd > 5 && idx.row() < 2 && m_threeDayViewMultiDayModels.count() < 3) {
+            return {};
+        }
+
+        if (!m_threeDayViewMultiDayModels.contains(startDate)) {
+            m_threeDayViewMultiDayModels[startDate] = generateMultiDayIncidenceModel(startDate, 3, 3);
+            m_threeDayViewMultiDayModels[startDate]->setFilters(MultiDayIncidenceModel::AllDayOnly | MultiDayIncidenceModel::MultiDayOnly);
+
+            m_liveThreeDayViewMultiDayModelKeys.enqueue(startDate);
+            cleanUpModels();
+        } else {
+            requeue(m_liveThreeDayViewMultiDayModelKeys, startDate);
+        }
+
+        return QVariant::fromValue(m_threeDayViewMultiDayModels[startDate]);
+    }
+    case DayViewModelRole: {
+        m_lastAccessedModelType = TypeDay;
+
+        if (m_datesToAdd > 5 && idx.row() < 2 && m_dayViewModels.count() < 3) {
+            return {};
+        }
+
+        if (!m_dayViewModels.contains(startDate)) {
+            m_dayViewModels[startDate] = generateHourlyIncidenceModel(startDate, 1, 15);
+
+            m_liveDayViewModelKeys.enqueue(startDate);
+            cleanUpModels();
+        } else {
+            requeue(m_liveDayViewModelKeys, startDate);
+        }
+
+        return QVariant::fromValue(m_dayViewModels[startDate]);
+    }
+    case DayViewMultiDayModelRole: {
+        m_lastAccessedModelType = TypeDayMultiDay;
+
+        if (m_datesToAdd > 5 && idx.row() < 2 && m_dayViewMultiDayModels.count() < 3) {
+            return {};
+        }
+
+        if (!m_dayViewMultiDayModels.contains(startDate)) {
+            m_dayViewMultiDayModels[startDate] = generateMultiDayIncidenceModel(startDate, 1, 1);
+            m_dayViewMultiDayModels[startDate]->setFilters(MultiDayIncidenceModel::AllDayOnly | MultiDayIncidenceModel::MultiDayOnly);
+
+            m_liveDayViewMultiDayModelKeys.enqueue(startDate);
+            cleanUpModels();
+        } else {
+            requeue(m_liveDayViewMultiDayModelKeys, startDate);
+        }
+
+        return QVariant::fromValue(m_dayViewMultiDayModels[startDate]);
+    }
     default:
         qCWarning(KALENDAR_LOG) << "Unknown role for startdate:" << QMetaEnum::fromType<Roles>().valueToKey(role);
         return {};
@@ -255,12 +371,22 @@ QHash<int, QByteArray> InfiniteCalendarViewModel::roleNames() const
         {ScheduleViewModelRole, QByteArrayLiteral("scheduleViewModel")},
         {WeekViewModelRole, QByteArrayLiteral("weekViewModel")},
         {WeekViewMultiDayModelRole, QByteArrayLiteral("weekViewMultiDayViewModel")},
+        {ThreeDayViewModelRole, QByteArrayLiteral("threeDayViewModel")},
+        {ThreeDayViewMultiDayModelRole, QByteArrayLiteral("threeDayViewMultiDayViewModel")},
+        {DayViewModelRole, QByteArrayLiteral("dayViewModel")},
+        {DayViewMultiDayModelRole, QByteArrayLiteral("dayViewMultiDayViewModel")},
     };
 }
 
 void InfiniteCalendarViewModel::addDates(bool atEnd, const QDate startFrom)
 {
     switch (m_scale) {
+    case DayScale:
+        addDayDates(atEnd, startFrom);
+        break;
+    case ThreeDayScale:
+        addDayDates(atEnd, startFrom, 3);
+        break;
     case WeekScale:
         addWeekDates(atEnd, startFrom);
         break;
@@ -276,6 +402,25 @@ void InfiniteCalendarViewModel::addDates(bool atEnd, const QDate startFrom)
     }
 }
 
+void InfiniteCalendarViewModel::addDayDates(bool atEnd, const QDate &startFrom, int amount)
+{
+    const int newRow = atEnd ? rowCount() : 0;
+
+    beginInsertRows(QModelIndex(), newRow, newRow + m_datesToAdd - 1);
+
+    for (int i = 0; i < m_datesToAdd; i++) {
+        QDate startDate = startFrom.isValid() && i == 0 ? startFrom : atEnd ? m_startDates[rowCount() - 1].addDays(amount) : m_startDates[0].addDays(-amount);
+
+        if (atEnd) {
+            m_startDates.append(startDate);
+        } else {
+            m_startDates.insert(0, startDate);
+        }
+    }
+
+    endInsertRows();
+}
+
 void InfiniteCalendarViewModel::addWeekDates(bool atEnd, const QDate &startFrom)
 {
     const int newRow = atEnd ? rowCount() : 0;
@@ -422,6 +567,22 @@ void InfiniteCalendarViewModel::setCalendar(Akonadi::ETMCalendar *calendar)
         model->model()->setCalendar(calendar);
     }
 
+    for (auto model : std::as_const(m_threeDayViewModels)) {
+        model->model()->setCalendar(calendar);
+    }
+
+    for (auto model : std::as_const(m_threeDayViewMultiDayModels)) {
+        model->model()->setCalendar(calendar);
+    }
+
+    for (auto model : std::as_const(m_dayViewModels)) {
+        model->model()->setCalendar(calendar);
+    }
+
+    for (auto model : std::as_const(m_dayViewMultiDayModels)) {
+        model->model()->setCalendar(calendar);
+    }
+
     Q_EMIT calendarChanged();
 }
 
@@ -445,5 +606,28 @@ void InfiniteCalendarViewModel::setFilter(const QVariantMap &filter)
     for (auto model : std::as_const(m_weekViewMultiDayModels)) {
         model->model()->setFilter(filter);
     }
+    for (auto model : std::as_const(m_threeDayViewModels)) {
+        model->model()->setFilter(filter);
+    }
+    for (auto model : std::as_const(m_threeDayViewMultiDayModels)) {
+        model->model()->setFilter(filter);
+    }
+    for (auto model : std::as_const(m_dayViewModels)) {
+        model->model()->setFilter(filter);
+    }
+    for (auto model : std::as_const(m_dayViewMultiDayModels)) {
+        model->model()->setFilter(filter);
+    }
     Q_EMIT filterChanged();
 }
+
+int InfiniteCalendarViewModel::maxLiveModels()
+{
+    return m_maxLiveModels;
+}
+
+void InfiniteCalendarViewModel::setMaxLiveModels(int maxLiveModels)
+{
+    m_maxLiveModels = maxLiveModels;
+    Q_EMIT maxLiveModelsChanged();
+}
