@@ -15,13 +15,14 @@ class InfiniteCalendarViewModel : public QAbstractListModel
     // Amount of dates to add each time the model adds more dates
     Q_PROPERTY(int datesToAdd READ datesToAdd WRITE setDatesToAdd NOTIFY datesToAddChanged)
     Q_PROPERTY(int scale READ scale WRITE setScale NOTIFY scaleChanged)
-    Q_PROPERTY(QStringList weekViewLocalisedHourLabels MEMBER m_weekViewLocalisedHourLabels CONSTANT)
+    Q_PROPERTY(QStringList hourlyViewLocalisedHourLabels MEMBER m_hourlyViewLocalisedHourLabels CONSTANT)
     Q_PROPERTY(Akonadi::ETMCalendar *calendar READ calendar WRITE setCalendar NOTIFY calendarChanged)
     Q_PROPERTY(QVariantMap filter READ filter WRITE setFilter NOTIFY filterChanged)
+    Q_PROPERTY(int maxLiveModels READ maxLiveModels WRITE setMaxLiveModels NOTIFY maxLiveModelsChanged)
 
 public:
     // The decade scale is designed to be used in a 4x3 grid, so shows 12 years at a time
-    enum Scale { WeekScale, MonthScale, YearScale, DecadeScale };
+    enum Scale { DayScale, ThreeDayScale, WeekScale, MonthScale, YearScale, DecadeScale };
     Q_ENUM(Scale);
 
     enum Roles {
@@ -32,11 +33,15 @@ public:
         MonthViewModelRole,
         ScheduleViewModelRole,
         WeekViewModelRole,
-        WeekViewMultiDayModelRole
+        WeekViewMultiDayModelRole,
+        ThreeDayViewModelRole,
+        ThreeDayViewMultiDayModelRole,
+        DayViewModelRole,
+        DayViewMultiDayModelRole
     };
     Q_ENUM(Roles);
 
-    enum ModelType { TypeMonth, TypeSchedule, TypeWeek, TypeWeekMultiDay };
+    enum ModelType { TypeDay, TypeDayMultiDay, TypeThreeDay, TypeThreeDayMultiDay, TypeMonth, TypeSchedule, TypeWeek, TypeWeekMultiDay };
     Q_ENUM(ModelType);
 
     explicit InfiniteCalendarViewModel(QObject *parent = nullptr);
@@ -48,6 +53,7 @@ public:
     int rowCount(const QModelIndex &parent = {}) const override;
 
     Q_INVOKABLE void addDates(bool atEnd, const QDate startFrom = QDate());
+    void addDayDates(bool atEnd, const QDate &startFrom, int amount = 1);
     void addWeekDates(bool atEnd, const QDate &startFrom);
     void addMonthDates(bool atEnd, const QDate &startFrom);
     void addYearDates(bool atEnd, const QDate &startFrom);
@@ -65,6 +71,12 @@ public:
     QVariantMap filter() const;
     void setFilter(const QVariantMap &filter);
 
+    int maxLiveModels();
+    void setMaxLiveModels(int maxLiveModels);
+
+    QDateTime openDate();
+    void setOpenDate(QDateTime openDate);
+
     void checkModels(const QDate &start, const QDate &end, KCalendarCore::Incidence::Ptr incidence);
     void checkCalendarIndex(const QModelIndex &index);
     void triggerAffectedModelUpdates();
@@ -77,11 +89,13 @@ Q_SIGNALS:
     void scaleChanged();
     void calendarChanged();
     void filterChanged();
+    void maxLiveModelsChanged();
+    void openDateChanged();
 
 private:
     QVector<QDate> m_startDates;
     QVector<QDate> m_firstDayOfMonthDates;
-    QStringList m_weekViewLocalisedHourLabels;
+    QStringList m_hourlyViewLocalisedHourLabels;
     QLocale m_locale;
     int m_datesToAdd = 10;
     int m_scale = MonthScale;
@@ -91,7 +105,7 @@ private:
         int modelLength;
         int modelType;
         QHash<QDate, MultiDayIncidenceModel *> *multiDayModels;
-        QHash<QDate, HourlyIncidenceModel *> *weekModels;
+        QHash<QDate, HourlyIncidenceModel *> *hourlyModels;
         QQueue<QDate> *liveKeysQueue;
     };
 
@@ -100,12 +114,20 @@ private:
     mutable QHash<QDate, MultiDayIncidenceModel *> m_scheduleViewModels;
     mutable QHash<QDate, HourlyIncidenceModel *> m_weekViewModels;
     mutable QHash<QDate, MultiDayIncidenceModel *> m_weekViewMultiDayModels;
+    mutable QHash<QDate, HourlyIncidenceModel *> m_threeDayViewModels;
+    mutable QHash<QDate, MultiDayIncidenceModel *> m_threeDayViewMultiDayModels;
+    mutable QHash<QDate, HourlyIncidenceModel *> m_dayViewModels;
+    mutable QHash<QDate, MultiDayIncidenceModel *> m_dayViewMultiDayModels;
     QSet<Akonadi::Item::Id> m_insertedIds;
     mutable QQueue<QDate> m_liveMonthViewModelKeys;
     mutable QQueue<QDate> m_liveScheduleViewModelKeys;
     mutable QQueue<QDate> m_liveWeekViewModelKeys;
     mutable QQueue<QDate> m_liveWeekViewMultiDayModelKeys;
-    int m_maxLiveModels = 8;
+    mutable QQueue<QDate> m_liveThreeDayViewModelKeys;
+    mutable QQueue<QDate> m_liveThreeDayViewMultiDayModelKeys;
+    mutable QQueue<QDate> m_liveDayViewModelKeys;
+    mutable QQueue<QDate> m_liveDayViewMultiDayModelKeys;
+    int m_maxLiveModels = 10;
     mutable int m_lastAccessedModelType = TypeMonth;
     Akonadi::ETMCalendar *m_calendar;
     QVariantMap mFilter;
