@@ -1,6 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Carl Schwan <carlschwan@kde.org>
 // SPDX-License-Identifier: LGPL-2.1-or-later
-
+#define QT_QML_DEBUG
 #include <KAboutData>
 #include <KCalendarCore/MemoryCalendar>
 #include <KCalendarCore/VCalFormat>
@@ -8,6 +8,7 @@
 #include <KLocalizedContext>
 #include <KLocalizedString>
 #include <KWindowConfig>
+#include <KWindowSystem>
 #include <QApplication>
 #include <QCommandLineParser>
 #include <QDBusConnection>
@@ -15,6 +16,7 @@
 #include <QDir>
 #include <QQmlApplicationEngine>
 #include <QQmlContext>
+#include <QQmlDebuggingEnabler>
 #include <QQuickStyle>
 #include <QQuickWindow>
 #include <QUrl>
@@ -43,9 +45,21 @@
 
 using namespace KCalendarCore;
 
+static void raiseWindow(QWindow *window)
+{
+    if (KWindowSystem::isPlatformWayland()) {
+        KWindowSystem::setCurrentXdgActivationToken(qEnvironmentVariable("XDG_ACTIVATION_TOKEN"));
+        KWindowSystem::activateWindow(window->winId());
+    } else {
+        window->raise();
+    }
+}
+
 int main(int argc, char *argv[])
 {
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     QGuiApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
+#endif
     QApplication app(argc, argv);
     KLocalizedString::setApplicationDomain("kalendar");
     QCoreApplication::setOrganizationName(QStringLiteral("KDE"));
@@ -114,6 +128,7 @@ int main(int argc, char *argv[])
     aboutData.processCommandLine(&parser);
 
     QQmlApplicationEngine engine;
+    QQmlDebuggingEnabler enabler;
 
     auto contactsManager = new ContactsManager(&engine);
     auto tagManager = new TagManager(&engine);
@@ -147,12 +162,23 @@ int main(int argc, char *argv[])
         return -1;
     }
 
+    QObject::connect(&service, &KDBusService::activateRequested, &engine, [&engine](const QStringList & /*arguments*/, const QString & /*workingDirectory*/) {
+        const auto rootObjects = engine.rootObjects();
+        for (auto obj : rootObjects) {
+            auto view = qobject_cast<QQuickWindow *>(obj);
+            if (view) {
+                view->show();
+                raiseWindow(view);
+                return;
+            }
+        }
+    });
     const auto rootObjects = engine.rootObjects();
     for (auto obj : rootObjects) {
         auto view = qobject_cast<QQuickWindow *>(obj);
         if (view) {
             KConfig dataResource(QStringLiteral("data"), KConfig::SimpleConfig, QStandardPaths::AppDataLocation);
-            KConfigGroup windowGroup(&dataResource, QStringLiteral("Window"));
+            KConfigGroup windowGroup(&dataResource, "Window");
             KWindowConfig::restoreWindowSize(view, windowGroup);
             KWindowConfig::restoreWindowPosition(view, windowGroup);
             break;
